# frozen_string_literal: true

# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Auto-generated by gapic-generator-ruby. DO NOT EDIT!


module Google
  module Cloud
    module Channel
      module V1
        # Entity representing a customer of a reseller or distributor.
        # @!attribute [r] name
        #   @return [::String]
        #     Output only. Resource name of the customer.
        #     Format: accounts/\\{account_id}/customers/\\{customer_id}
        # @!attribute [rw] org_display_name
        #   @return [::String]
        #     Required. Name of the organization that the customer entity represents.
        # @!attribute [rw] org_postal_address
        #   @return [::Google::Type::PostalAddress]
        #     Required. Address of the organization of the customer entity.
        #     Region and zip codes are required to enforce US laws and embargoes.
        #     Valid address lines are required for all customers.
        #     Language code is discarded. Use the Customer-level language code to set the
        #     customer's language.
        # @!attribute [rw] primary_contact_info
        #   @return [::Google::Cloud::Channel::V1::ContactInfo]
        #     Primary contact info.
        # @!attribute [rw] alternate_email
        #   @return [::String]
        #     Secondary contact email.
        #     Alternate email and primary contact email are required to have different
        #     domains if primary contact email is present.
        #     When creating admin.google.com accounts, users get notified credentials at
        #     this email. This email address is also used as a recovery email.
        # @!attribute [rw] domain
        #   @return [::String]
        #     Required. Primary domain used by the customer.
        #     Domain of primary contact email is required to be same as the provided
        #     domain.
        # @!attribute [r] create_time
        #   @return [::Google::Protobuf::Timestamp]
        #     Output only. The time at which the customer is created.
        # @!attribute [r] update_time
        #   @return [::Google::Protobuf::Timestamp]
        #     Output only. The time at which the customer is updated.
        # @!attribute [r] cloud_identity_id
        #   @return [::String]
        #     Output only. Customer's cloud_identity_id.
        #     Populated only if a Cloud Identity resource exists for this customer.
        # @!attribute [rw] language_code
        #   @return [::String]
        #     Optional. The BCP-47 language code, such as "en-US" or "sr-Latn". For more
        #     information, see
        #     https://www.unicode.org/reports/tr35/#Unicode_locale_identifier.
        # @!attribute [r] cloud_identity_info
        #   @return [::Google::Cloud::Channel::V1::CloudIdentityInfo]
        #     Output only. Cloud Identity information for the customer.
        #     Populated only if a Cloud Identity account exists for this customer.
        # @!attribute [rw] channel_partner_id
        #   @return [::String]
        #     Cloud Identity ID of the customer's channel partner.
        #     Populated only if a channel partner exists for this customer.
        class Customer
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end

        # Contact information for a customer account.
        # @!attribute [rw] first_name
        #   @return [::String]
        #     First name of the contact in the customer account.
        # @!attribute [rw] last_name
        #   @return [::String]
        #     Last name of the contact in the customer account.
        # @!attribute [r] display_name
        #   @return [::String]
        #     Output only. Display name of the contact in the customer account.
        #     Populated by combining customer first name and last name.
        # @!attribute [rw] email
        #   @return [::String]
        #     Email of the contact in the customer account.
        #     Email is required for entitlements that need creation of admin.google.com
        #     accounts. The email will be the username used in credentials to access the
        #     admin.google.com account.
        # @!attribute [rw] title
        #   @return [::String]
        #     Optional. Job title of the contact in the customer account.
        # @!attribute [rw] phone
        #   @return [::String]
        #     Phone number of the contact in the customer account.
        class ContactInfo
          include ::Google::Protobuf::MessageExts
          extend ::Google::Protobuf::MessageExts::ClassMethods
        end
      end
    end
  end
end
